"""
TODO:
  * Gør elev og lærer id til id men hvor at elev id har et S foran (Eks: S123456789) og lærer har et T foran (Eks: T12345678)
  * Fjern retry fra enkelte funktioner og gør det til en general ting
  * Få lærere, elever, lokaler, osv. fra https://www.lectio.dk/lectio/681/FindSkemaAdv.aspx
"""

from .imports import *
from . import _auth, _lektier, _skema, _modul, _opgaver, _beskeder, _informationer, _filer, _fravær

class sdk:
    def __init__(self, brugernavn, adgangskode, skoleId, base64Cookie=None):
        self.session = requests.session()

        if base64Cookie == None:
            self.brugernavn = brugernavn
            self.adgangskode = adgangskode
            self.skoleId = skoleId

            self.login()
        else:
            cookie = json.loads(base64.b64decode(base64Cookie))
            self.skoleId = cookie["LastLoginExamno"]
            self.elevId = cookie["LastLoginElevId"]

            for identifier, value in cookie.items():
                self.session.cookies.set(identifier, value, domain="lectio.dk")

    def login(self):
        return _auth.login(self)

    def base64Cookie(self):
        return _auth.base64Cookie(self)

    def lektier(self):
        return _lektier.lektier(self)

    def skema(self, retry=False, uge=None, år=None, elevId=None):
        return _skema.skema(self, retry=retry, uge=uge, år=år, elevId=elevId)

    def modul(self, absid):
        return _modul.modul(self, absid)

    def opgave(self, exerciseid):
        return _opgaver.opgave(self, exerciseid)
    def opgaver(self):
        return _opgaver.opgaver(self)

    def besked(self, message_id):
        return _beskeder.besked(self, message_id=message_id)

    def beskeder(self, id=None):
        return _beskeder.beskeder(self, id=id)

    def informationer(self):
        return _informationer.informationer(self)

    def fåElev(self, elevId):
        return _informationer.fåElev(self, elevId)

    def fåBruger(self, brugerId):
        return _informationer.fåBruger(self, brugerId)

    def fåFil(self, filUrl):
        return _filer.fåFil(self, filUrl)

    def fravær(self):
        return _fravær.fravær(self)


from .imports import *

renameDictionary = {
    "Lærere": "Lærer",
    "Lokaler": "Lokale"
}
def fravær(self):
    resp = self.session.get(f"https://www.lectio.dk/lectio/{self.skoleId}/subnav/fravaerelev.aspx?elevid={self.elevId}")
    soup = BeautifulSoup(resp.text, "html.parser")

    fravær = {
        "generalt": [],
        "moduler": {
            "manglende_fraværsårsager": [],
            "oversigt": []
        },
        "grafisk_oversigt": "https://www.lectio.dk" + soup.find("img", {"class": "fravaer_billede"}).get("src")
    }

    for hold in soup.find("table", {"id": "s_m_Content_Content_SFTabStudentAbsenceDataTable"}).find_all("tr"):
        if hold.get("id") == None:
            row = hold.find_all("td")
            fravær["generalt"].append({
                "hold": row[0].text,
                "fravær_procent": row[1].text,
                "fravær_moduler": row[2].text,
            })

    resp = self.session.get(f"https://www.lectio.dk/lectio/{self.skoleId}/subnav/fravaerelev_fravaersaarsager.aspx?elevid={self.elevId}")
    soup = BeautifulSoup(resp.text, "html.parser")

    manglende = soup.find("table", {"id": "s_m_Content_Content_FatabMissingAarsagerGV"})
    if "Der er ingen manglende fraværsårsager..." not in str(manglende):
        for tr in manglende.find_all("tr")[1:]:
            modul = tr.find("a", class_="s2skemabrik")
            modulDetaljer = modul["data-additionalinfo"].split("\n\n")[0].split("\n")

            modulDict = {
                "navn": None,
                "tidspunkt": None,
                "hold": None,
                "lærer": None,
                "lokale": None,
                "absid": re.search('absid=[0-9]+', modul["href"]).group().replace("absid=", "")
            }
            for modulDetalje in modulDetaljer:
                if (value := ": ".join(modulDetalje.split(": ")[1:])) != "":
                    if (navn := modulDetalje.split(": ")[0]) in renameDictionary:
                        navn = renameDictionary[navn]

                    modulDict[navn.lower()] = value
                else:
                    try:
                        int(datetime.strptime(modulDetalje.split(": ")[0].split(" til")[0],
                                              "%d/%m-%Y %H:%M").timestamp())
                        modulDict["tidspunkt"] = modulDetalje
                    except Exception:
                        modulDict["navn"] = modulDetalje.split(": ")[0]

            tds = tr.find_all("td")

            fravær["moduler"]["manglende_fraværsårsager"].append({
                "type": tds[0].text.lstrip(),
                "uge": tds[1].text.lstrip(),
                "aktivitet": modulDict,
                "fravær": tds[3].text.lstrip(),
                "fraværstype": tds[4].text.lstrip(),
                "registreret": tds[5].text.lstrip(),
                "lærer": tds[6].text.lstrip(),
                "bemærkning": None
            })

    oversigt = soup.find("table", {"id": "s_m_Content_Content_FatabAbsenceFravaerGV"})
    if "Der er ikke indtastet nogen fraværsårsager" not in str(oversigt):
        for tr in oversigt.find_all("tr")[1:]:
            modul = tr.find("a", class_="s2skemabrik")
            modulDetaljer = modul["data-additionalinfo"].split("\n\n")[0].split("\n")

            modulDict = {
                "navn": None,
                "tidspunkt": None,
                "hold": None,
                "lærer": None,
                "lokale": None,
                "absid": re.search('absid=[0-9]+', modul["href"]).group().replace("absid=", "")
            }
            for modulDetalje in modulDetaljer:
                if (value := ": ".join(modulDetalje.split(": ")[1:])) != "":
                    if (navn := modulDetalje.split(": ")[0]) in renameDictionary:
                        navn = renameDictionary[navn]

                    modulDict[navn.lower()] = value
                else:
                    try:
                        int(datetime.strptime(modulDetalje.split(": ")[0].split(" til")[0],
                                              "%d/%m-%Y %H:%M").timestamp())
                        modulDict["tidspunkt"] = modulDetalje
                    except Exception:
                        modulDict["navn"] = modulDetalje.split(": ")[0]

            tds = tr.find_all("td")

            fravær["moduler"]["oversigt"].append({
                "type": tds[0].text.lstrip(),
                "uge": tds[1].text.lstrip(),
                "aktivitet": modulDict,
                "fravær": tds[3].text.lstrip(),
                "fraværstype": tds[4].text.lstrip(),
                "registreret": tds[5].text.lstrip(),
                "lærer": tds[6].text.lstrip(),
                "bemærkning": tds[7].text.lstrip(),
                "årsag": tds[8].text.lstrip(),
                "årsagsnote": tds[9].text.lstrip(),
            })

    return fravær



from .imports import *

def modul(self, absid):
    resp = self.session.get(
        f"https://www.lectio.dk/lectio/{self.skoleId}/aktivitet/aktivitetforside2.aspx?absid={absid}")
    soup = BeautifulSoup(resp.text, "html.parser")

    modulDetaljer = {
        "aktivitet": None,
        "note": None,
        "lektier": "",
        "øvrigtIndhold": ""
    }

    try:
        modulDetaljer["note"] = soup.find("textarea", {"class": "activity-note"}).text.lstrip()
    except Exception:
        pass

    modulContent = soup.find("div", {"id": "s_m_Content_Content_tocAndToolbar_inlineHomeworkDiv"})
    last = ""
    for div in modulContent.find_all("div"):
        if div.get("style") == None:
            if (divText := div.text.lstrip().rstrip()) != "":
                last = divText.lower().title().replace(" ", "")
                last = last[0].lower() + last[1:]
        else:
            for element in str(div).split("\n"):
                elementSoup = BeautifulSoup(element, "html.parser")
                if elementSoup.text != "":
                    if (elementWithHref := elementSoup.find("a", href=True)) != None:
                        href = elementWithHref.get('href')
                        if href.startswith(f"/lectio/{self.skoleId}"):
                            href = "https://www.lectio.dk" + href
                        modulDetaljer[last] += unicodedata.normalize("NFKD",
                                                                     f"[{elementSoup.text.rstrip().lstrip()}]({href})\n")
                    else:
                        modulDetaljer[last] += unicodedata.normalize("NFKD",
                                                                     elementSoup.text.rstrip().lstrip().replace(u"\xa0",
                                                                                                                u" ") + "\n")

    renameDictionary = {
        "Lærere": "Lærer",
        "Lokaler": "Lokale"
    }

    modulDict = {
        "navn": None,
        "tidspunkt": None,
        "hold": None,
        "lærer": None,
        "lokale": None,
        "absid": absid
    }
    for modulDetalje in soup.find("a", class_="s2skemabrik")["data-additionalinfo"].split("\n\n")[0].split("\n"):
        if (value := ": ".join(modulDetalje.split(": ")[1:])) != "":
            if (navn := modulDetalje.split(": ")[0]) in renameDictionary:
                navn = renameDictionary[navn]

            modulDict[navn.lower()] = value
        else:
            try:
                int(datetime.strptime(modulDetalje.split(": ")[0].split(" til")[0],
                                      "%d/%m-%Y %H:%M").timestamp())
                modulDict["tidspunkt"] = modulDetalje
            except Exception:
                modulDict["navn"] = modulDetalje.split(": ")[0]

    modulDetaljer["aktivitet"] = modulDict

    return modulDetaljer

def elevFeedback():
    pass


from .imports import *

def login(self):
    headers = {
        'accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.9',
        'accept-encoding': 'gzip, deflate, br',
        'accept-language': 'en-DK,en;q=0.9',
        'sec-ch-ua': '" Not A;Brand";v="99", "Chromium";v="101", "Google Chrome";v="101"',
        'sec-ch-ua-mobile': '?0',
        'sec-ch-ua-platform': '"Linux"',
        'sec-fetch-dest': 'document',
        'sec-fetch-mode': 'navigate',
        'sec-fetch-site': 'none',
        'sec-fetch-user': '?1',
        'upgrade-insecure-requests': '1',
        'user-agent': 'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/101.0.0.0 Safari/537.36',
    }
    resp = self.session.get(f"https://www.lectio.dk/lectio/{self.skoleId}/login.aspx", headers=headers)
    soup = BeautifulSoup(resp.text, "html.parser")
    payload = {}
    for payloadItem in soup.find_all({"input": {"type": "hidden"}}):
        payload[payloadItem.get('name')] = payloadItem.get('value')

    del payload["query"]
    del payload["m$defaultformbtn"]

    payload["__EVENTTARGET"] = "m$Content$submitbtn2"
    payload["m$Content$username"] = self.brugernavn
    payload["m$Content$password"] = self.adgangskode
    payload["m$Content$AutologinCbx"] = "on"
    payload["LectioPostbackId"] = ""

    headers = {
        'accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.9',
        'accept-encoding': 'gzip, deflate, br',
        'accept-language': 'en-DK,en;q=0.9',
        'cache-control': 'max-age=0',
        'content-length': str(len(json.dumps(payload, separators=(',', ':')))),
        'content-type': 'application/x-www-form-urlencoded',
        'origin': 'https://www.lectio.dk',
        'referer': f'https://www.lectio.dk/lectio/{self.skoleId}/login.aspx',
        'sec-ch-ua': '" Not A;Brand";v="99", "Chromium";v="101", "Google Chrome";v="101"',
        'sec-ch-ua-mobile': '?0',
        'sec-ch-ua-platform': '"Linux"',
        'sec-fetch-dest': 'document',
        'sec-fetch-mode': 'navigate',
        'sec-fetch-site': 'same-origin',
        'sec-fetch-user': '?1',
        'upgrade-insecure-requests': '1',
        'user-agent': 'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/101.0.0.0 Safari/537.36',
    }
    resp = self.session.post(f"https://www.lectio.dk/lectio/{self.skoleId}/login.aspx", data=payload, headers=headers)
    soup = BeautifulSoup(resp.text, "html.parser")

    successful = False
    for meta in soup.find_all({"meta": {"name": "msapplication-starturl"}}):
        if f"/lectio/{self.skoleId}/forside.aspx?" in str(meta.get("content")):
            self.elevId = meta.get("content").split("?elevid=")[1]
            successful = True
            break
    if not successful:
        raise Exception("Kunne ikke finde elev id. Rapporter venligst dette på Github")


def base64Cookie(self):
    cookie = self.session.cookies.get_dict()
    cookie["LastLoginElevId"] = self.elevId

    return base64.b64encode(json.dumps(cookie).encode())


from .imports import *

def skema(self, retry=False, uge=None, år=None, elevId=None):
    if elevId == None:
        elevId = self.elevId

    if uge == None and år == None:
        resp = self.session.get(f"https://www.lectio.dk/lectio/{self.skoleId}/SkemaNy.aspx?type=elev&elevid={elevId}")
    elif uge != None and år != None:
        uge = str(uge)
        år = str(år)
        if len(uge) == 1:
            uge = "0" + uge
        resp = self.session.get(
            f"https://www.lectio.dk/lectio/{self.skoleId}/SkemaNy.aspx?type=elev&elevid={elevId}&week={uge}{år}")
    else:
        raise Exception("Enten skal hverken uge og år være i brug ellers skal både uge og år være i brug")

    soup = BeautifulSoup(resp.text, "html.parser")

    skema = {
        "modulTider": {},
        "ugeDage": [],
        "moduler": [],
        "dagsNoter": []
    }

    for modulTid in soup.find_all("div", {"class": "s2module-info"}):
        skema["modulTider"][modulTid.prettify().split("\n")[2].lstrip()] = modulTid.prettify().split("\n")[4].lstrip()

    for dag in soup.find("tr", {"class": "s2dayHeader"}).find_all("td"):
        if dag.text != "":
            skema["ugeDage"].append(dag.text)

    i = 0
    for dagsNoter in soup.find_all("td", {"class": "s2infoHeader s2skemabrikcontainer"}):
        skema["dagsNoter"].append({
            skema["ugeDage"][i]: []
        })
        for dagsNote in dagsNoter.find_all("a"):
            skema["dagsNoter"][i][skema["ugeDage"][i]].append(dagsNote.text.lstrip())
        i += 1

    successful = False
    i = 0

    renameDictionary = {
        "Lærere": "Lærer",
        "Lokaler": "Lokale"
    }

    statusDictionary = {
        "s2brik": "normal",
        "s2cancelled": "aflyst",
        "s2changed": "ændret",

        "s2bgboxeksamen": "eksamen"
    }

    for dag in soup.find_all("div", class_="s2skemabrikcontainer"):
        if i != 0:
            dag = BeautifulSoup(str(dag), "html.parser")
            for modul in dag.find_all("a", class_="s2skemabrik"):
                try:
                    absid = re.search('absid=[0-9]+', modul["href"]).group().replace("absid=", "")
                except Exception:
                    absid = modul.get("href")

                modulDict = {
                    "navn": None,
                    "tidspunkt": None,
                    "hold": None,
                    "lærer": None,
                    "lokale": None,
                    "status": "normal",
                    "absid": absid,
                    "andet": None
                }

                modulDetaljer = modul
                statusClass = modulDetaljer.get("class")[2]
                if statusClass in statusDictionary:
                    modulDict["status"] = statusDictionary[modulDetaljer.get("class")[2]]
                else:
                    modulDict["status"] = modulDetaljer.get("class")[2]

                modulDetaljer = modulDetaljer["data-additionalinfo"].split("\n\n")[0].split("\n")

                for modulDetalje in modulDetaljer:
                    if (value := ": ".join(modulDetalje.split(": ")[1:])) != "":
                        if (navn := modulDetalje.split(": ")[0]) in renameDictionary:
                            navn = renameDictionary[navn]

                        modulDict[navn.lower()] = value
                    else:
                        try:
                            int(datetime.strptime(modulDetalje.split(": ")[0].split(" til")[0],
                                                  "%d/%m-%Y %H:%M").timestamp())
                            modulDict["tidspunkt"] = modulDetalje
                        except Exception:
                            modulDict["navn"] = modulDetalje.split(": ")[0]

                try:
                    modulDict["andet"] = modul["data-additionalinfo"].split("\n\n")[1]
                except Exception:
                    pass

                skema["moduler"].append(modulDict)
        i += 1

    if len(skema) > 0:
        successful = True

    if successful:
        return skema
    elif not retry:
        self.login()
        return self.skema(retry=True)
    else:
        return False


from .imports import *

def opgave(self, exerciseid):
    resp = self.session.get(f"https://www.lectio.dk/lectio/{self.skoleId}/ElevAflevering.aspx?elevid={self.elevId}&exerciseid={exerciseid}")
    soup = BeautifulSoup(resp.text, "html.parser")

    opgaveDict = {
        "oplysninger": {
            "opgavetitel": None,
            "opgavebeskrivelse": None,
            "opgavenote": None,
            "hold": None,
            "karakterskala": None,
            "ansvarlig": None,
            "elevtid": None,
            "afleveringsfrist": None,
            "i_undervisningsbeskrivelse": None
        },
        "gruppemedlemmer": [],
        "afleveres_af": {},
        "opgave_indlæg": []
    }
    for tr in soup.find("table", {"class": "ls-std-table-inputlist"}).find_all("tr"):
        if (identifier := unicodedata.normalize("NFKD", tr.find("th").text).lower().replace(" ", "_").replace(":", "")) == "ansvarlig":
            opgaveDict["oplysninger"][identifier] = {"navn": unicodedata.normalize("NFKD", tr.find("td").text), "bruger_id": tr.find("span").get("data-lectiocontextcard")}
        elif identifier == "opgavebeskrivelse":
            opgaveBeskrivelse = ""
            for a in tr.find_all("a"):
                opgaveBeskrivelse += f'[{unicodedata.normalize("NFKD", a.text).lstrip().rstrip()}](https://www.lectio.dk{a.get("href")})\n  '
            opgaveDict["oplysninger"][identifier] = opgaveBeskrivelse[:-1]
        else:
            opgaveDict["oplysninger"][identifier] = unicodedata.normalize("NFKD", tr.find("td").text)

    if soup.find_all("span", {"class": "islandHeader"})[1].text == "Gruppeaflevering":
        for tr in soup.find("table", {"class": "ls-table-layout1 lf-grid"}).find_all("tr")[1:]:
            opgaveDict["gruppemedlemmer"].append({"navn": unicodedata.normalize("NFKD", tr.text.lstrip().rstrip()), "bruger_id": tr.find("span").get("data-lectiocontextcard")})

    header = soup.find("table", {"class": "ls-table-layout1 maxW textTop lf-grid"}).find_all("tr", {"class": ""})[0]
    headerIdentifiers = []
    for th in header.find_all("th")[1:]:
        headerIdentifiers.append(th.text.lower().replace(" ", "_"))

    for tr in soup.find("table", {"class": "ls-table-layout1 maxW textTop lf-grid"}).find_all("tr")[1:]:
        i = 0
        for td in tr.find_all("td")[1:]:
            if (identifier := headerIdentifiers[i]) == "afsluttet":
                opgaveDict["afleveres_af"][identifier] = td.find("input").get("checked") == "checked"
            elif identifier == "elev":
                opgaveDict["afleveres_af"][identifier] = {"navn": unicodedata.normalize("NFKD", td.text).lstrip().rstrip(), "bruger_id": td.find_all("span")[1].get("data-lectiocontextcard")}
            elif identifier == "status_-_frav\u00e6r":
                opgaveDict["afleveres_af"]["status_frav\u00e6r"] = unicodedata.normalize("NFKD", td.text).lstrip().rstrip()
            
            else:
                opgaveDict["afleveres_af"][identifier] = unicodedata.normalize("NFKD", td.text).lstrip().rstrip()
            i += 1

    indlægHtml = soup.find("table", {"id": "m_Content_RecipientGV"})
    indlægHeader = [header.text.lower().replace(" ", "_") for header in indlægHtml.find_all("th")]

    for tr in indlægHtml.find_all("tr")[1:]:
        indlæg = {}
        i = 0
        for td in tr.find_all("td"):
            if (identifier := indlægHeader[i]) == "dokument" and (href := td.find("a").get('href')) != None:
                indlæg[identifier] = f"[{td.text.lstrip().rstrip()}](https://www.lectio.dk{href})"
            elif identifier == "bruger":
                indlæg[identifier] = {"navn": td.text.lstrip().rstrip(), "bruger_id": td.find("span").get("data-lectiocontextcard")}
            else:
                indlæg[identifier] = td.text.lstrip().rstrip()
            i += 1

        opgaveDict["opgave_indlæg"].append(indlæg)

    return opgaveDict
def opgaver(self):
    resp = self.session.get(f"https://www.lectio.dk/lectio/{self.skoleId}/OpgaverElev.aspx?elevid={self.elevId}")
    soup = BeautifulSoup(resp.text, "html.parser")

    opgaver = []
    header = []

    _header = soup.find("tr")
    for th in _header.find_all("th"):
        header.append(th.text.lower().replace("\xad", "-"))

    for opgave in soup.find_all("tr")[1:]:
        opgaveDict = {}

        i = 0
        for td in opgave.find_all("td"):
            if header[i] == "opgavetitel":
                opgaveDict["exerciseid"] = re.findall("exerciseid=\d+", td.find("a", href=True).get("href"))[0].replace(
                    "exerciseid=", "")

            opgaveDict[header[i]] = td.text.lstrip()
            i += 1

        opgaver.append(opgaveDict)

    return opgaver

def afleverOpgave(self, exerciseid, file=None, note=None):
    pass



from .imports import *

def informationer(self):
    resp = self.session.get(f"https://www.lectio.dk/lectio/{self.skoleId}/FindSkemaAdv.aspx")
    soup = BeautifulSoup(resp.text, "html.parser")

    informationerDict = {}

    for section in soup.find_all("section", {"class": "island"}):
        sectionName = section.find("span", {"class": "islandHeader"}).text.lower().replace(" ", "_")
        informationerDict[sectionName] = {}
        select = section.find("select")
        if select != None:
            for option in select.find_all("option"):
                informationerDict[sectionName][option.text] = option.get("value")

    return informationerDict

def fåBruger(self, brugerId):
    resp = self.session.get(f"https://www.lectio.dk/lectio/{self.skoleId}/contextcard/contextcard.aspx?lectiocontextcard={brugerId}")
    soup = BeautifulSoup(resp.text, "html.parser")

    titel = soup.find("span").text

    bruger = {
        "navn": titel.split(" - ")[-1],
        "pictureid": None,
        "type": titel.split(" - ")[0].lower()
    }

    try:
        bruger["pictureid"] = re.search("pictureid=\d+", soup.find("img").get("src")).group().replace("pictureid=", "")
    except AttributeError:
        pass

    if bruger["type"] == "lærer":
        bruger["hold"] = {}
        for hold in soup.find("table", {"class": "textTop"}).find_all("a"):
            bruger["hold"][hold.text] = re.search("holdelementid=\d+", hold.get("href")).group().replace("holdelementid=", "")
    else:
        bruger["stamklasse"] = soup.find("table", {"class": "textTop"}).find_all("td")[1].text

    return bruger

def fåElev(self, elevId):
    resp = self.session.get(f"https://www.lectio.dk/lectio/{self.skoleId}/SkemaNy.aspx?type=elev&elevid={elevId}")
    soup = BeautifulSoup(resp.text, "html.parser")

    elev = {
        "navn": soup.find("div", {"class": "maintitle"}).text.replace("Eleven ", "").replace(" - Skema", ""),
        "billede": soup.find("img", {"id": "s_m_HeaderContent_picctrlthumbimage"}).get("src")
    }

    return elev


from .imports import *

def lektier(self):
    resp = self.session.get(f"https://www.lectio.dk/lectio/{self.skoleId}/material_lektieoversigt.aspx?elevid={self.elevId}")

    soup = BeautifulSoup(resp.text, "html.parser")

    lektier = []

    renameDictionary = {
        "Lærere": "Lærer",
        "Lokaler": "Lokale"
    }

    for tr in soup.find_all("tr"):
        modul = tr.find("a", class_="s2skemabrik")
        modulDetaljer = modul["data-additionalinfo"].split("\n\n")[0].split("\n")

        modulDict = {
            "navn": None,
            "tidspunkt": None,
            "hold": None,
            "lærer": None,
            "lokale": None,
            "absid": re.search('absid=[0-9]+', modul["href"]).group().replace("absid=", "")
        }
        for modulDetalje in modulDetaljer:
            if (value := ": ".join(modulDetalje.split(": ")[1:])) != "":
                if (navn := modulDetalje.split(": ")[0]) in renameDictionary:
                    navn = renameDictionary[navn]

                modulDict[navn.lower()] = value
            else:
                try:
                    int(datetime.strptime(modulDetalje.split(": ")[0].split(" til")[0],
                                          "%d/%m-%Y %H:%M").timestamp())
                    modulDict["tidspunkt"] = modulDetalje
                except Exception:
                    modulDict["navn"] = modulDetalje.split(": ")[0]

        lektie = {
            "dato": tr.find("th").get("b"),
            "aktivitet": modulDict,
            "note": tr.find_all("td")[1].text,
            "lektier": {
                "beskrivelse": tr.find("td", {"class": "ls-homework"}).find("a").text,
                "link": tr.find("td", {"class": "ls-homework"}).find("a").get("href")
            }
        }

        lektier.append(lektie)

    return lektier


from setuptools import setup, find_packages
from pathlib import Path

VERSION = '1.6.3'
DESCRIPTION = 'En SDK til gymnasie siden Lectio'
long_description = (Path(__file__).parent / "README.md").read_text()

# Setting up
setup(
    name="python-lectio",
    version=VERSION,
    author="jona799t",
    #author_email="<not@available.com>",
    url = 'https://github.com/jona799t/python-lectio',
    description=DESCRIPTION,
    long_description=long_description,
    license_files = ('LICENSE.txt',),
    long_description_content_type='text/markdown',
    packages=find_packages(),
    install_requires=["requests", "beautifulsoup4", "markdownify"],
    keywords=['python', 'lectio', 'sdk', 'gymnasie', 'gymnasium'],
    classifiers=[
        "Development Status :: 1 - Planning",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Operating System :: Unix",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
    ]
)



from .imports import *

def beskeder(self, id=None):
    if id != None:
        if "-" in str(id):
            url = f"https://www.lectio.dk/lectio/{self.skoleId}/beskeder2.aspx?type=&elevid={self.elevId}&selectedfolderid={id}"
        else:
            url = f"https://www.lectio.dk/lectio/{self.skoleId}/beskeder2.aspx?type=selecthold&elevid={self.elevId}&holdid={id}"
    else:
        url = f"https://www.lectio.dk/lectio/680/beskeder2.aspx?type=liste&elevid={self.elevId}"

    resp = self.session.get(url)
    soup = BeautifulSoup(resp.text, "html.parser")

    options = []
    for div in soup.find("div", {"id": "s_m_Content_Content_ListGridSelectionTree"}).find_all("div", {
        "lec-role": "treeviewnodecontainer"}):
        if "-" in div.get("lec-node-id"):
            name = div.text.rstrip()
            option = {
                "name": None,
                "id": div.get("lec-node-id"),
                "selected": False,
                "content": []
            }
            if div.find("a", {"class": "selectedFolder"}) != None:
                option["selected"] = True

            if len(name.split("\n")) == 1:
                option["name"] = name
                options.append(option)
            else:
                name = div.text.rstrip().split("\n")[0]
                option["name"] = name
                for item in div.find_all("div", {"lec-role": "treeviewnodecontainer"}):
                    option["content"].append({
                        "name": item.text.rstrip(),
                        "id": item.get("lec-node-id")
                    })

                options.append(option)

    beskederHtml = soup.find_all("tr")

    beskedHeader = []
    for th in beskederHtml[0].find_all("th"):
        if len(header := th.text.rstrip()) != 0:
            header = header.lower().title().replace(" ", "")
            header = header[0].lower() + header[1:]
            beskedHeader.append(header)

    beskeder = []
    for beskedHtml in beskederHtml[1:]:
        besked = {}

        i = 0
        for td in beskedHtml.find_all("td"):
            if len(value := td.text.lstrip().rstrip()) != 0:
                if beskedHeader[i] == "modtagere":
                    besked[beskedHeader[i]] = td.find("span").get("title").split("\r\n")
                else:
                    try:
                        besked[beskedHeader[i]] = td.find("span").get("title")
                    except Exception:
                        besked[beskedHeader[i]] = value

                    if beskedHeader[i] == "emne":
                        besked["message_id"] = re.findall("\$_\d+", td.find("a").get("onclick"))[0].replace("$_", "")

                i += 1

        beskeder.append(besked)

    return {"besked_muligheder": options, "beskeder": beskeder}

def besked(self, message_id):
    resp = self.session.get(f"https://www.lectio.dk/lectio/681/beskeder2.aspx?type=showthread&elevid={self.elevId}&id={message_id}")
    soup = BeautifulSoup(resp.text, "html.parser")

    beskeder = []
    beskederHtml = soup.find("ul", {"id": "s_m_Content_Content_ThreadList"}).find_all("li")

    for besked in beskederHtml:
        bruger = besked.find("span")
        beskedDiv = None
        for div in besked.find_all("div"):
            if div.get("class") == None:
                beskedDiv = div
                break

        if beskedDiv == None:
            raise Exception("Kan ikke finde besked, rapporter venligst dette på Github")

        beskedDict = {
            "bruger": {"navn": bruger.text, "id": bruger.get("data-lectiocontextcard")},
            "titel": besked.find("h4").text,
            "dato": re.search("[0-9]+/[0-9]+-[0-9]+ [0-9]+:[0-9]+", besked.find("td").text).group(),
            "padding_left": re.search("padding-left:\d*\.?\d*", str(besked.get("style"))).group().replace("padding-left:", ""),
            "besked": markdownify.markdownify(str(beskedDiv), heading_style="ATX").lstrip().rstrip().replace("\n\n", "\n"),
            "vedhæftninger": [{"navn": vedhæft.text, "href": "https://www.lectio.dk"+vedhæft.get("href")} for vedhæft in besked.find("td").find_all("a", {"id": None})]
        }
        beskeder.append(beskedDict)

    return beskeder

def sendBesked(self, message_id, content):
    pass